% Barone-Adesi and Whaley quadratic approximation to vanilla options
% by Fabrice Douglas Rouah, FRouah.com and Volopta.com

clc; clear;

% Input the option parameters
S = 50;                    % Spot Price
K = 50;                    % Strike Price
T = 5/12;                  % Maturity
r = 0.05;                  % Risk free rate
q = 0.02;                  % Dividend yield
v = 0.15;                  % Volatility
PutCall = 'P';             % 'C'all or 'P'ut

% Define a range of spot prices
S = [40:55];

% Find the European and American prices, and the critical stock value S*
fprintf(' Spot       Euror     Amer    Premium\n')
fprintf('--------------------------------------- \n')
for i = 1:length(S)
	[Sx(i) EuroPrice(i) AmerPrice(i)] = BAWPrice(S(i),K,r,q,v,T,PutCall);
    fprintf('%8.4f  %8.4f  %8.4f %8.4f\n',...
        S(i),EuroPrice(i),AmerPrice(i),AmerPrice(i)-EuroPrice(i));
end
fprintf('--------------------------------------- \n')

% Define the payoff
if strcmp(PutCall,'C')
	Payoff = max(S-K,0);
else
	Payoff = max(K-S,0);
end

%% Show that the American value and payoff start to diverge at S*
I = find(AmerPrice-Payoff==0);
J = (I(end)-3:I(end)+4);
Difference = AmerPrice(J)' - Payoff(J)';
fprintf('AmericanPrice  Payoff    Difference   Spot     Sx \n')
fprintf('-------------------------------------------------------\n')
for j=1:length(J)
    fprintf('%8.4f %12.4f  %8.4f  %11.4f %8.4f \n',...
        AmerPrice(J(j)),Payoff(J(j)),Difference(j),S(J(j)),Sx(J(j)));
end
fprintf('-------------------------------------------------------\n')

% Plot the results
plot(S,EuroPrice,'k',S,AmerPrice,'r',S,Payoff,'k:')
hold on
line([Sx(1); Sx(1)], [0 max(AmerPrice)])
legend('European Value', 'American Value', 'Value at Expiry', 'S* for Early Exercise')
hold off
