function y = findSx(Sx,K,r,q,v,T,PutCall)
% Finds the critical stock price, Sx, above/below which it becomes optimal
% to exercise American options.

% by Fabrice Douglas Rouah, FRouah.com and Volopta.com

% Anonymous functions for Black Scholes call and put
BSCall = @(s,K,r,q,v,T) s.*exp(-q.*T).*normcdf((log(s./K) + (r-q+v.^2./2).*T)./v./sqrt(T)) - K.*exp(-r.*T).*normcdf((log(s./K) + (r-q+v.^2./2).*T)./v./sqrt(T) - v.*sqrt(T));
BSPut  = @(s,K,r,q,v,T) K.*exp(-r.*T).*normcdf(-(log(s./K) + (r-q+v.^2./2).*T)./v./sqrt(T) + v.*sqrt(T)) - s.*exp(-q.*T).*normcdf(-(log(s./K) + (r-q+v.^2./2).*T)./v./sqrt(T));

n = 2*(r-q)/v^2;
k = 2*r/v^2/(1-exp(-r*T));

if Sx < 0
	y = 1e100;
elseif strcmp(PutCall,'C')
	q2 = (1-n+sqrt((n-1)^2+4*k))/2;
	y = (BSCall(Sx,K,r,q,v,T) ...
		+ (1 - exp(-q*T)*normcdf((log(Sx/K) + (r-q+v^2/2))/v/sqrt(T)))*Sx/q2 ...
		- Sx + K)^2;
else
	q1 = (1-n-sqrt((n-1)^2+4*k))/2;
	y = (BSPut(Sx,K,r,q,v,T) ...
		- (1 - exp(-q*T)*normcdf(-(log(Sx/K) + (r-q+v^2/2))/v/sqrt(T)))*Sx/q1 ...
		+ Sx - K)^2;
end;	

